import cv2
import argparse

from hampy import detect_markers

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('-c', '--camera', type=int, default=-1)
    args = parser.parse_args()

    cap = cv2.VideoCapture(args.camera)

    while True:
        okay, img = cap.read()
        if not okay:
            continue

        markers = detect_markers(img)

        for m in markers:
            m.draw_contour(img)

            cv2.putText(img, str(m.id), tuple(int(p) for p in m.center),
                        cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 3)

        cv2.imshow('live', img)
        cv2.waitKey(20)



import cv2

from numpy import array, all, zeros, ones, rot90
from skimage.measure import find_contours
from matplotlib.mlab import find

from .hamming_marker import HammingMarker, marker_size
from .hamming import decode


def sk2cv(contours):
    l = []

    for c in contours:
        ll = zeros((c.shape[0], 1, 2), dtype='int32')
        ll[:, 0, 0] = c[:, 1]
        ll[:, 0, 1] = c[:, 0]

        l.append(array(ll))
    return l


def detect_markers(img, marker_ids=None):
    width, height, _ = img.shape

    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    contours = sk2cv(find_contours(gray, 50))

    # We only keep the big enough contours
    min_area = width * height * .01
    contours = [c for c in contours if cv2.contourArea(c) > min_area]

    warped_size = 9 * 10
    canonical_marker_coords = array(((0, 0),
                                     (warped_size - 1, 0),
                                     (warped_size - 1, warped_size - 1),
                                     (0, warped_size - 1)),
                                    dtype='float32')

    markers = []

    for c in contours:
        approx_curve = cv2.approxPolyDP(c, len(c) * 0.01, True)
        if not (len(approx_curve) == 4 and cv2.isContourConvex(approx_curve)):
            continue

        sorted_curve = array(cv2.convexHull(approx_curve, clockwise=False),
                             dtype='float32')
        persp_transf = cv2.getPerspectiveTransform(sorted_curve,
                                                   canonical_marker_coords)

        warped_img = cv2.warpPerspective(img, persp_transf,
                                         (warped_size, warped_size))
        warped_gray = cv2.cvtColor(warped_img, cv2.COLOR_BGR2GRAY)
        _, warped_bin = cv2.threshold(warped_gray, 50, 255, cv2.THRESH_BINARY)

        marker = warped_bin.reshape([marker_size,
                                     warped_size / marker_size,
                                     marker_size,
                                     warped_size / marker_size])
        marker = marker.mean(axis=3).mean(axis=1)
        marker[marker < 127] = 0
        marker[marker >= 127] = 1

        # Eliminate the entirely black or entirely white markers
        # for robustness purposes
        sub_marker = marker[1:-1, 1:-1]
        sub_size = marker_size - 2
        if (all(sub_marker == zeros((sub_size, sub_size))) or
                all(sub_marker == ones((sub_size, sub_size)))):
            continue

        for _ in range(4):
            try:
                code = decode(sub_marker).flatten()
                id = (2 ** find(code == 1)).sum()
                markers.append(HammingMarker(id=id, contours=approx_curve, img_size=(width, height)))
            except ValueError:  # The hamming code is incorrect
                pass

            sub_marker = rot90(sub_marker)

    # Remove duplicates
    markers = {m.id: m for m in markers}.values()

    return markers



import cv2
import argparse

from hampy import HammingMarker

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--id', type=int)
    parser.add_argument('--size', type=int, default=64)
    args = parser.parse_args()

    m = HammingMarker(args.id) if args.id else HammingMarker.generate()
    img = (1 - m.toimage(args.size)) * 255

    cv2.imshow('marker_{}'.format(m.id), img)
    cv2.imwrite('marker_{}.png'.format(m.id), img)
    cv2.waitKey(-1)



import cv2

from numpy import array, mean, binary_repr, zeros
from numpy.random import randint
from scipy.ndimage import zoom
from PIL import Image

from hampy.hamming import encode, msg_size, data_size

marker_size = msg_size + 2
max_id = 2 ** (msg_size * data_size)


class HammingMarker(object):
    def __init__(self, id, contours=None, img_size=None):
        if not 0 <= id < max_id:
            msg = 'Id must be {} <= id < {} (id={})'.format(0, max_id, id)
            raise ValueError(msg)

        self.id = id
        self.hamming_code = self._encode_id(self.id)
        self.contours = contours
        self.size = list(reversed(img_size))

    def __repr__(self):
        return '<Marker id={} center={}>'.format(self.id, self.center)

    @property
    def center(self):
        if self.contours is None:
            return None

        return mean(self.contours, axis=0).flatten()

    @property
    def normalized_center(self):
        return ((array(self.center, dtype=float) / self.size) - [0.5, 0.5]) * 2

    def toimage(self, size=marker_size, output=None):
        img = zeros((marker_size, marker_size))
        img[1:-1, 1:-1] = self.hamming_code
        img = 1 - img

        # scale = size / float(marker_size)
        # return zoom(img, zoom=scale, output=output, order=0)

        cv2.imwrite('/tmp/bob.png', img)
        im = Image.open('/tmp/bob.png')
        im2 = im.resize((size, size))
        im2.save('/tmp/bob.png')
        return cv2.imread('/tmp/bob.png')

    def draw_contour(self, img, color=(0, 255, 0), linewidth=5):
        cv2.drawContours(img, [self.contours], -1, color, linewidth)

    @classmethod
    def generate(cls):
        return HammingMarker(id=randint(max_id))

    def _encode_id(self, id):
        s = binary_repr(id, width=msg_size * data_size)
        B = array(list(s), dtype=int).reshape(msg_size, data_size)
        return encode(B)



from numpy import array, dot

hamming_code = (7, 4)
msg_size, data_size = hamming_code

parity_bits = [0, 1, 3]
data_bits = [2, 4, 5, 6]

G = array(((1, 1, 0, 1),
           (1, 0, 1, 1),
           (1, 0, 0, 0),
           (0, 1, 1, 1),
           (0, 1, 0, 0),
           (0, 0, 1, 0),
           (0, 0, 0, 1)))

H = array(((0, 0, 0, 1, 1, 1, 1),
           (0, 1, 1, 0, 0, 1, 1),
           (1, 0, 1, 0, 1, 0, 1)))


def encode(B):
    """ Encode data using Hamming(7, 4) code.

    E.g.:
        encode([0, 0, 1, 1])

        encode([[0, 0, 0, 1],
                [0, 1, 0, 1]])

    :param array B: binary data to encode (must be shaped as (4, ) or (-1, 4)).

    """
    B = array(B)

    flatten = False
    if len(B.shape) == 1:
        flatten = True
        B = B.reshape(1, -1)

    if B.shape[1] != data_size:
        raise ValueError('Data must be shaped as (4, ) or (-1, 4)')

    C = dot(G, B.T).T % 2

    if flatten:
        C = C.flatten()

    return C


def decode(C):
    """ Decode data using Hamming(7, 4) code.

    E.g.:
        decode([1, 0, 0, 0, 0, 1, 1])

        encode([[1, 1, 0, 1, 0, 0, 1],
                [0, 1, 0, 0, 1, 0, 1]])

    :param array C: binary data to code (must be shaped as (7, ) or (-1, 7)).

    """
    C = array(C)

    flatten = False
    if len(C.shape) == 1:
        flatten = True
        C = C.reshape(1, -1)

    if C.shape[1] != msg_size:
        raise ValueError('Data must be shaped as (7, ) or (-1, 7)')

    if 1 in dot(H, C.T).T % 2:
        raise ValueError('Incorrect code given as input.')

    B = C[:, data_bits]

    if flatten:
        B = B.flatten()

    return B


if __name__ == '__main__':
    import unittest

    from numpy import all
    from numpy.random import randint

    class TestHammingCodeDecode(unittest.TestCase):
        def setUp(self):
            self.b1 = array([0, 0, 1, 1])
            self.b2 = [[0, 0, 0, 1],
                       [0, 1, 0, 1]]

            self.c1 = [1, 0, 0, 0, 0, 1, 1]
            self.c2 = array([[1, 1, 0, 1, 0, 0, 1],
                             [0, 1, 0, 0, 1, 0, 1]])

        def test_code(self):
            self.assertTrue(all(encode(self.b1) == self.c1))
            self.assertTrue(all(encode(self.b2) == self.c2))

        def test_decode(self):
            decode(self.c2)

        def test_code_decode(self):
            B = randint(2, size=(1000, data_size))
            self.assertTrue(all(decode(encode(B)) == B))

    unittest.main()



#!/usr/bin/env python

from setuptools import setup, find_packages


setup(name='hampy',
      version='1.3.1',
      packages=find_packages(),

      install_requires=['numpy', 'matplotlib', 'scipy', 'scikit-image'],
      setup_requires=['setuptools_git >= 0.3', ],

      author='Pierre Rouanet',
      author_email='pierre.rouanet@gmail.com',
      description='Simple Hamming Marker Detection using OpenCV',
      url='https://github.com/pierre-rouanet/hampy',
      license='GNU GENERAL PUBLIC LICENSE Version 3',
      )
